env1 <- function(obj) {
  res <- resid(obj); n <- length(res)
  H <- model.matrix(obj); p <- ncol(H)
  res <- res/sqrt(sum(res^2)/(n-p))
  H <- H %*% solve(t(H) %*% H)  %*% t(H)
  ident <- diag(n)
  epsilon <- matrix(0, n, 1000)
  e <- matrix(0, n, 1000)
  e1 <- numeric(n); e2 <- numeric(n)
  for(i in 1:1000) {
    epsilon[, i] <- rnorm(n, 0, 1)
    e[, i] <- (ident - H) %*% epsilon[, i]
    e[, i] <- sort(e[, i])/sqrt(sum(e[, i]^2)/(n-p))
  }
  for(i in 1:n) {
    eo <- quantile(e[i, ], c(0.025, 0.975))
    e1[i] <- eo[1]; e2[i] <- eo[2]
  }
  ylim <- range(res, e1, e2)
  oldpar <- par(pty = "s"); on.exit(par(oldpar))
  qqnorm(res, pch = 1, ylim = ylim, ylab = "Std. Residuals",
         main = deparse(obj$call$formula))
  par(new = T)
  qqnorm(e1, axes=F, xlab="", ylab="", type="l", ylim=ylim)
  par(new = T)
  qqnorm(e2, axes=F, xlab="", ylab="", type="l", ylim=ylim)
  invisible(obj)
}

env2 <- function(object, nsamples = 1000, alpha = 0.05) {
  n <- length(r <- resid(object))
  p <- length(coef(object))
  Y <- scale(qr.resid(qr(model.matrix(object)),
         matrix(rnorm(n * nsamples), n, nsamples)),
                      F, T) * sqrt((n - 1)/(n - p))
  Y[, ] <- Y[order(col(Y), Y)]
  Y <- matrix(Y[order(row(Y), Y)], nsamples, n)
  x0 <- quantile(1:nsamples, c(alpha/2, 1-alpha/2))
  if(all(x0 %% 1 == 0)) elim <- t(Y[x0, ])
  else {
    x1 <- c(floor(x0), ceiling(x0))
    elim <- cbind(
          Y[x1[1], ] + (Y[x1[3], ] - Y[x1[1], ])/
          (x1[3] - x1[1])*(x0[1] - x1[1]),
          Y[x1[2], ] + (Y[x1[4], ] - Y[x1[2], ])/
          (x1[4] - x1[2])*(x0[2] - x1[2]))
  }
  res <- sort(r)
  res <- res/sqrt(sum(res^2)/(n-p))
  res <- structure(list(res=res, elim=elim),
            label = deparse(object$call$formula))
  class(res) <- "envelope" # or oldClass
  res
}

plot.envelope <- function(x, ...) {
  n <- length(x$res)
  ylim <- range(x$res[c(1,n)], x$elim[c(1,n),])
  nscores <- qnorm(ppoints(n))
  oldpar <- par(pty = "s"); on.exit(par(oldpar))
  plot(nscores, x$res, pch = 1, ylim = ylim,
       xlab = "Normal scores", ylab = "Sorted residuals",
       main = attr(x, "label"))
  lines(nscores, x$elim[,1]); lines(nscores, x$elim[,2])
  invisible(x)
}

print.envelope <- function(x, ...) {
  lo <- x$res < x$elim[,1]; hi <- x$res > x$elim[,2]
  flash <- rep("", length(lo))
  flash[lo] <- "<"; flash[hi] <- ">"
  if(any(lo | hi))
    print(cbind(do.call("data.frame",x), flash=flash)[lo | hi,])
  else cat("All points within envelope\n")
  invisible(x)
}

library(MASS)
fm <- lm(log(volume)~log(diam)+log(height), trees)
resources(env1(fm))

print(resources(plot(env2(fm))))
