guiCreate("FunctionInfo", Name = "menuLDA",
          Function = "menuLDA",
          HelpCommand = "help(lda)",
          DialogHeader = "Linear Discriminant Analysis",
          PropertyList = c("ldaModelPage", "ldaPlotPage",
            "ldaPredictPage", "SPropPFEnableButton"),
          CallBackFunction = "backLDA",
          ArgumentList = "#0=ldaSaveAs, #1=SPropPFFormula,
  #2=SPropDataFrameList, #3=SPropSubset, #4=SPropOmitMissing,
  #5=ldaFitMethod, #6=SPropPredictNewdata,
  #7=SPropSavePredictObject, #8=ldaPredictMethod,
  #9=ldaPlot, #10=ldaPlotDimen, #11=ldaCV, #12=ldaFitNu")

backLDA <- function(df)
{
  activeprop <- cbGetActiveProp(df)
  activevalue <- cbGetCurrValue(df, activeprop)
  switch(activeprop,
    ldaPlot = {
       df <- cbSetEnableFlag(df, "ldaPlotDimen",
                               activevalue=="T")
    },
    ldaFitMethod = {
      if(activevalue == "moment" || activevalue == "mle") {
        df <- cbSetEnableFlag(df, "ldaCV", T)
      } else {
        df <- cbSetEnableFlag(df, "ldaCV", F)
        df <- cbSetCurrValue(df, "ldaCV", "F")
      }
      df <- cbSetEnableFlag(df, "ldaFitNu", activevalue=="t")
    },
    ldaCV = {
      set <- activevalue == "F"
      df <- cbSetEnableFlag(df, "ldaPlot", set)
      df <- cbSetEnableFlag(df, "ldaPredictMethod", set)
      df <- cbSetEnableFlag(df, "SPropPredictNewdf", set)
      df <- cbSetEnableFlag(df, "SPropSavePredictObject", set)
    },
    SPropPFButton = backform1(df),
    SPropPFEnableButton = {
      df <- cbSetEnableFlag(df, "SPropDataFrameList", T)
      df <- cbSetEnableFlag(df, "SPropPFFormula", T)
      df <- cbSetEnableFlag(df, "SPropPFButton", T)
    }
  )
  df
}

backform1 <- function(df)
{
  if(exists(cbGetCurrValue(df, "SPropDataFrameList"))) {
    guiModify("Property", Name = "SPropCFFormulaID",
              DialogControl = "Invisible",
              DefaultValue = cbGetDialogId(df))
    guiModify("Property", Name = "SPropCFData",
              DialogControl = "Invisible",
       DefaultValue = cbGetCurrValue(df, "SPropDataFrameList"))
    guiModify("Property", Name = "SPropCFFormula",
       DefaultValue = cbGetCurrValue(df, "SPropPFFormula"))
    guiDisplayDialog("Function", Name = "makeFormulaLm")
    df <- cbSetEnableFlag(df, "SPropDataFrameList", F)
    df <- cbSetEnableFlag(df, "SPropPFFormula", F)
    df <- cbSetEnableFlag(df, "SPropPFButton", F)
  } else guiCreate("MessageBox", String = "Data Frame doesn't
            exist. Enter Data Frame before building formula.")
}

    # Properties for the dialog box
    guiCreate("Property", Name="ldaModelPage", Type="Page",
              DialogPrompt="Model",
              PropertyList=paste("ldaModelData, SPropFSpace1, SPropFSpace2",
              "SPropPFFormulaG", "ldaMethod, ldaSaveModel"))
    guiCreate("Property", Name="ldaModelData", Type="Group",
              DialogPrompt="Data",
              PropertyList="SPropDataFrameList, SPropSubset,
              SPropOmitMissing")
    guiCreate("Property", Name="ldaMethod", Type="Group",
              DialogPrompt="Fit Method",
              PropertyList="ldaFitMethod, ldaCV, ldaFitNu")
    guiCreate("Property", Name="ldaSaveModel", Type="Group",
              DialogPrompt="Save Model Object",
              PropertyList="ldaSaveAs")
    guiCreate("Property", Name="ldaFitMethod",
              DialogPrompt="Method",
              DialogControl="List Box",
              OptionList="moment, mle, mve, t",
              DefaultValue="moment", UseQuotes=T)
    guiCreate("Property", Name="ldaCV",
              DialogPrompt="Cross-validation (leave 1 out)",
              DialogControl="Check Box", DefaultValue=F)
    guiCreate("Property", Name="ldaFitNu",
              DialogPrompt="df for t fit",
              DialogControl="Integer Spinner", Range=2:50,
              DefaultValue=5, Disable=T)
    guiCreate("Property", Name="ldaSaveAs", DialogPrompt="Save As",
              DialogControl="String", DefaultValue="last.lda")

    guiCreate("Property", Name="ldaPredictPage", Type="Page",
              DialogPrompt="Predict",
              PropertyList="SPropPredictNewdata, ldaPredictMethod,
              SPropSavePredictObject")
    guiCreate("Property", Name="ldaPredictMethod",
              DialogPrompt="Method",
              DialogControl="List Box",
              OptionList="plug-in, predictive, debiased",
              DefaultValue="plug-in", UseQuotes=T)

    guiCreate("Property", Name="ldaPlotPage", Type="Page",
              DialogPrompt="Plot",
              PropertyList="ldaPlot, ldaPlotDimen")
    guiCreate("Property", Name="ldaPlot", DialogPrompt="Plot LDs",
              DialogControl="Check Box", DefaultValue=F)
    guiCreate("Property", Name="ldaPlotDimen", DialogPrompt="Dimen",
              DialogControl="Integer Spinner", Range=1:10,
              DefaultValue=3, Disable=T)

